@@ -193,6 +193,13 @@ def translate_MDSserver(server, treename):
     return server
 
 
+def mds_connection_cache_ID_generator(action, server, treename, shot):
+    """
+    This function returns a tuple that is used determine if a MDS+ connection should be reused
+    """
+    return action, server
+
+
 def tunneled_MDSserver(MDSserver, quiet=True, forceRemote=False):
     if ':' not in MDSserver:
         raise ValueError('MDS server `%s` is not in the format hostname:port' % MDSserver)
@@ -526,7 +533,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
             pre = ''
             post = ''
 
-        mds_connection_cache_ID = ('read', server, treename, shot)
+        mds_connection_cache_ID = mds_connection_cache_ID_generator('read', server, treename, shot)
 
         # try twice, the second time reconnect to server
         for fallback in range(2):
@@ -728,7 +735,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
 
         :param quantities: list of quantities to write
         """
-        mds_connection_cache_ID = ('write', server, treename, shot)
+        mds_connection_cache_ID = mds_connection_cache_ID_generator('write', server, treename, shot)
         if mds_connection_cache_ID not in self:
             self[mds_connection_cache_ID] = MDSplus.Connection(server)
             self[mds_connection_cache_ID].openTree(treename, shot)
@@ -785,7 +792,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         if translator is None:
             translator = lambda x: x
 
-        mds_connection_cache_ID = ('write', server, treename, shot)
+        mds_connection_cache_ID = mds_connection_cache_ID_generator('write', server, treename, shot)
         if mds_connection_cache_ID not in self:
             self[mds_connection_cache_ID] = MDSplus.Connection(server)
             # create pulse from model tree
@@ -902,7 +909,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         self.tcl(server, 'close')
 
     def tcl(self, server, tcl):
-        mds_connection_cache_ID = ('tcl', server, None, None)
+        mds_connection_cache_ID = mds_connection_cache_ID_generator('tcl', server, None, None)
 
         # try twice, the second time reconnect to server
         for fallback in range(2):
@@ -947,6 +954,19 @@ def OMFITmdsConnectionBase(*args, **kw):
     return _OMFITmdsConnectionBaseObjects[tmp]
 
 
+def close_mds_connections():
+    """
+    MDS+ does not have a way to close connections (essentially these are closed when they are garbage collected)
+    This can lead to many connections to stay open on the server side.
+    This function deletes the connection from the OMFIT connections cache, and calls the garbage collector.
+    """
+    if len(_OMFITmdsConnectionBaseObjects):
+        for tmp in list(_OMFITmdsConnectionBaseObjects.keys()):
+            _OMFITmdsConnectionBaseObjects[tmp].clear()
+        _OMFITmdsConnectionBaseObjects.clear()
+        gc.collect()
+
+
 class OMFITcaching(SortedDict):
     def __init__(self, type, cachesDir=True, limit=1000):
         """
@@ -1256,8 +1276,8 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
         return self._MDSconn()
 
     # ---------
-    def _cached_fetch(self, mds_connection_cache_ID, TDI):
-        action, server0, treename0, shot0, var = mds_connection_cache_ID
+    def _cached_fetch(self, variable_cache_ID, TDI):
+        action, server0, treename0, shot0, var = variable_cache_ID
         ID = (server0, shot0, treename0, var, TDI)
         data = _special1
         building_cache = False
@@ -1279,16 +1299,16 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
             else:
                 printd('MDS cache miss: %s' % (ID,), topic='MDS')
             tunneled_server = tunneled_MDSserver(server0, quiet=self.quiet)
-            connection_cache_key = (action, tunneled_server, treename0, shot0)
+            mds_connection_cache_ID = mds_connection_cache_ID_generator(action, tunneled_server, treename0, shot0)
 
-            if connection_cache_key not in list(self.MDSconn.keys()) or not hasattr(self, '_data'):
+            if mds_connection_cache_ID not in list(self.MDSconn.keys()) or not hasattr(self, '_data'):
                 # connect to MDS+
                 data = self.MDSconn.mdsvalue(tunneled_server, treename0, shot0, TDI, timeout=self.timeout)
                 printd('* Connect and retrieve data ' + TDI, topic='MDS')
             else:
                 # already connected
                 try:
-                    data = mdsplus_data(self.MDSconn[connection_cache_key].get(TDI, timeout=self.timeout))
+                    data = mdsplus_data(self.MDSconn[mds_connection_cache_ID].get(TDI, timeout=self.timeout))
                     if self.treename is None and data is not None and len(data) == 1 and data[0] == 0:
                         raise Exception('Try local PTDATA')
                     printd('* Retrieving remote data ' + TDI, topic='MDS')
@@ -1321,11 +1341,11 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
         if not hasattr(self, 'server0'):
             self.interpret_id()
         var = omfit_hash(str((self.server0, self.treename0, self.shot0, self.TDI)), 10)
-        mds_connection_cache_ID = ('read', self.server0, self.treename0, self.shot0, var)
+        variable_cache_ID = ('read', self.server0, self.treename0, self.shot0, var)
 
         # fetch data
         if not hasattr(self, '_data') or self._data is _special1:
-            tmp, building_cache = self._cached_fetch(mds_connection_cache_ID, '_s{}={}'.format(var, self.TDI))
+            tmp, building_cache = self._cached_fetch(variable_cache_ID, '_s{}={}'.format(var, self.TDI))
 
             if tmp is _special1:
                 if not self.quiet:
@@ -1342,11 +1362,11 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
                     self._fetch_error()
                     self._fetch_error_dim_of()
 
-        return mds_connection_cache_ID, var
+        return variable_cache_ID, var
 
     # DIM_OF
     def _fetch_dim_of(self):
-        mds_connection_cache_ID, var = self._fetch_data()
+        variable_cache_ID, var = self._fetch_data()
         if not hasattr(self, '_data') or self._data is _special1:
             return
 
@@ -1355,7 +1375,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
             for k in range(len(self._data.shape)):
                 self._dim_of[k] = None
                 try:
-                    self._dim_of[k], building_cache = self._cached_fetch(mds_connection_cache_ID, 'dim_of(_s' + var + ',' + str(k) + ')')
+                    self._dim_of[k], building_cache = self._cached_fetch(variable_cache_ID, 'dim_of(_s' + var + ',' + str(k) + ')')
                     if self._dim_of[k] is not None:
                         self._dim_of[k] = np.atleast_1d(self._dim_of[k])
                 except Exception:
@@ -1363,20 +1383,20 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
     # UNITS
     def _fetch_units(self):
-        mds_connection_cache_ID, var = self._fetch_data()
+        variable_cache_ID, var = self._fetch_data()
         if not hasattr(self, '_data') or self._data is _special1:
             return
 
         if not hasattr(self, '_units'):
             self._units = None
             try:
-                self._units, building_cache = self._cached_fetch(mds_connection_cache_ID, 'units(_s' + var + ')')
+                self._units, building_cache = self._cached_fetch(variable_cache_ID, 'units(_s' + var + ')')
             except Exception:
                 pass
 
     # UNITS_DIM_OF
     def _fetch_units_dim_of(self):
-        mds_connection_cache_ID, var = self._fetch_data()
+        variable_cache_ID, var = self._fetch_data()
         if not hasattr(self, '_data') or self._data is _special1:
             return
 
@@ -1385,21 +1405,21 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
             for k in range(len(self._data.shape)):
                 try:
                     self._units_dim_of[k], building_cache = self._cached_fetch(
-                        mds_connection_cache_ID, 'units(dim_of(_s' + var + ',' + str(k) + '))'
+                        variable_cache_ID, 'units(dim_of(_s' + var + ',' + str(k) + '))'
                     )
                 except Exception:
                     pass
 
     # ERROR
     def _fetch_error(self):
-        mds_connection_cache_ID, var = self._fetch_data()
+        variable_cache_ID, var = self._fetch_data()
         if not hasattr(self, '_data') or self._data is _special1:
             return
 
         if not hasattr(self, '_error'):
             self._error = None
             try:
-                self._error, building_cache = self._cached_fetch(mds_connection_cache_ID, 'error_of(_s' + var + ')')
+                self._error, building_cache = self._cached_fetch(variable_cache_ID, 'error_of(_s' + var + ')')
                 if self._error is not None:
                     self._error = np.atleast_1d(self._error)
             except Exception:
@@ -1407,7 +1427,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
     # ERROR_DIM_OF
     def _fetch_error_dim_of(self):
-        mds_connection_cache_ID, var = self._fetch_data()
+        variable_cache_ID, var = self._fetch_data()
         self._fetch_error()
         if not hasattr(self, '_data') or self._data is _special1:
             return
@@ -1419,7 +1439,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
                     self._error_dim_of[k] = None
                     try:
                         self._error_dim_of[k], building_cache = self._cached_fetch(
-                            mds_connection_cache_ID, 'error_of(dim_of(_s' + var + ',' + str(k) + '))'
+                            variable_cache_ID, 'error_of(dim_of(_s' + var + ',' + str(k) + '))'
                         )
                         if self._error_dim_of[k] is not None:
                             self._error_dim_of[k] = np.atleast_1d(self._error_dim_of[k])
